@@ -19,9 +19,7 @@ export class Analysis {
             if (analysis.embedding) {
                 this.embedding = analysis.embedding.map(embedding => new Embedding(embedding));
             }
-            this.eigenValues = analysis.eigenValues;
-            this.baseEmbeddingName = analysis.baseEmbeddingName;
-            this.baseEmbeddingAxesIndices = analysis.baseEmbeddingAxesIndices;
+            this.eigenvalues = analysis.eigenvalues;
         }
     }
 
@@ -53,15 +51,5 @@ export class Analysis {
     /**
      * Contains the eigen values of the analysis.
      */
-    public eigenValues?: Array<number>;
-
-    /**
-     * Contains the name of the embedding on which the embedding was based.
-     */
-    public baseEmbeddingName?: string;
-
-    /**
-     * Contains a list of indices of the axes of the base embedding on which the embedding was based.
-     */
-    public baseEmbeddingAxesIndices?: Array<number>;
+    public eigenvalues?: Array<number>;
 }
