@@ -44,7 +44,7 @@ class Project:
 
         # Loads the project from the YAML file
         working_directory = os.path.dirname(self.path)
-        with open(self.path, 'r') as project_file:
+        with open(self.path, 'r', encoding='utf-8') as project_file:
             try:
 
                 # Loads the project and extracts some general information
@@ -57,7 +57,7 @@ class Project:
                 self.label_map = LabelMap(os.path.join(working_directory, project['label_map']))
 
                 # Loads the dataset of the project
-                if project['dataset'] is not None:
+                if 'dataset' in project:
                     dataset_type = project['dataset']['type']
                     if dataset_type == 'hdf5':
                         self.dataset = Hdf5Dataset(
@@ -78,10 +78,10 @@ class Project:
                             self.label_map
                         )
                     else:
-                        raise ValueError('The specified dataset type "{0}" is unknown.'.format(dataset_type))
+                        raise ValueError(f'The specified dataset type "{dataset_type}" is unknown.')
 
                 # Loads the attributions of the project
-                if project['attributions'] is not None:
+                if 'attributions' in project:
                     self.attribution_method = project['attributions']['attribution_method']
                     self.attribution_strategy = project['attributions']['attribution_strategy']
                     for attribution_database in project['attributions']['sources']:
@@ -91,7 +91,7 @@ class Project:
                         ))
 
                 # Loads the analyses of the project
-                if project['analyses'] is not None:
+                if 'analyses' in project:
                     for analysis in project['analyses']:
                         analysis_method = analysis['analysis_method']
                         if analysis_method not in self.analyses:
@@ -101,8 +101,8 @@ class Project:
                                 os.path.join(working_directory, analysis_database),
                                 self.label_map
                             ))
-            except yaml.YAMLError:
-                raise ValueError('An error occurred while loading the project file.')
+            except yaml.YAMLError as yaml_error:
+                raise ValueError('An error occurred while loading the project file.') from yaml_error
 
     def get_sample(self, index):
         """
@@ -115,6 +115,9 @@ class Project:
 
         Raises
         ------
+            ValueError
+                If the project has already been closed, then a ValueError is raised.
+                If the project does not contain a dataset, then a ValueError is raised.
             LookupError
                 If no dataset sample with the specified index could be found, then a LookupError is raised.
 
@@ -124,6 +127,12 @@ class Project:
                 Returns the dataset sample with the specified index.
         """
 
+        if self.is_closed:
+            raise ValueError('The project has already been closed.')
+
+        if self.dataset is None:
+            raise ValueError('The project does not contain a dataset.')
+
         return self.dataset.get_sample(index)
 
     def get_attribution(self, index):
@@ -137,6 +146,8 @@ class Project:
 
         Raises
         ------
+            ValueError
+                If the project has already been closed, then a ValueError is raised.
             LookupError
                 If no attribution with the specified index could be found, then a LookupError is raised
 
@@ -146,35 +157,48 @@ class Project:
                 Returns the attribution with the specified index.
         """
 
+        if self.is_closed:
+            raise ValueError('The project has already been closed.')
+
         for attribution_database in self.attributions:
             if attribution_database.has_attribution(index):
                 return attribution_database.get_attribution(index)
 
-        raise LookupError('No attribution with the specified index {0} could be found.'.format(index))
+        raise LookupError(f'No attribution with the specified index {index} could be found.')
 
     def get_analysis_methods(self):
         """
         Retrieves the names of all the analysis methods that are in this project.
 
+        Raises
+        ------
+            ValueError
+                If the project has already been closed, then a ValueError is raised.
+
         Returns
         -------
             list of str
                 Returns a list of the names of the all the analysis methods in this project.
         """
 
+        if self.is_closed:
+            raise ValueError('The project has already been closed.')
+
         return list(self.analyses.keys())
 
     def get_analysis_categories(self, analysis_method):
         """
         Retrieves the names of the categories that are in the analyses of the specified analysis method.
 
-        Parameter
-        ---------
+        Parameters
+        ----------
             analysis_method: str
                 The name of the analysis method for which the categories are to be retrieved.
 
         Raises
         ------
+            ValueError
+                If the project has already been closed, then a ValueError is raised.
             LookupError
                 If the specified analysis method does not exist, then a LookupError is raised.
 
@@ -184,8 +208,11 @@ class Project:
                 Returns a list of the names of the categories.
         """
 
+        if self.is_closed:
+            raise ValueError('The project has already been closed.')
+
         if analysis_method not in self.analyses:
-            raise LookupError('The specified analysis method "{0}" could not be found.'.format(analysis_method))
+            raise LookupError(f'The specified analysis method "{analysis_method}" could not be found.')
 
         categories = []
         for analysis in self.analyses[analysis_method]:
@@ -199,13 +226,15 @@ class Project:
         """
         Retrieves the names of the clustering methods that are in the analyses of the specified analysis method.
 
-        Parameter
-        ---------
+        Parameters
+        ----------
             analysis_method: str
                 The name of the analysis method for which the clusterings are to be retrieved.
 
         Raises
         ------
+            ValueError
+                If the project has already been closed, then a ValueError is raised.
             LookupError
                 If the specified analysis method does not exist, then a LookupError is raised.
 
@@ -215,8 +244,11 @@ class Project:
                 Returns a list of the names of the clusterings.
         """
 
+        if self.is_closed:
+            raise ValueError('The project has already been closed.')
+
         if analysis_method not in self.analyses:
-            raise LookupError('The specified analysis method "{0}" could not be found.'.format(analysis_method))
+            raise LookupError(f'The specified analysis method "{analysis_method}" could not be found.')
 
         return self.analyses[analysis_method][0].get_clustering_names()
 
@@ -224,13 +256,15 @@ class Project:
         """
         Retrieves the names of the embedding methods that are in the analyses of the specified analysis method.
 
-        Parameter
-        ---------
+        Parameters
+        ----------
             analysis_method: str
                 The name of the analysis method for which the embeddings are to be retrieved.
 
         Raises
         ------
+            ValueError
+                If the project has already been closed, then a ValueError is raised.
             LookupError
                 If the specified analysis method does not exist, then a LookupError is raised.
 
@@ -240,8 +274,11 @@ class Project:
                 Returns a list of the names of the embeddings.
         """
 
+        if self.is_closed:
+            raise ValueError('The project has already been closed.')
+
         if analysis_method not in self.analyses:
-            raise LookupError('The specified analysis method "{0}" could not be found.'.format(analysis_method))
+            raise LookupError(f'The specified analysis method "{analysis_method}" could not be found.')
 
         return self.analyses[analysis_method][0].get_embedding_names()
 
@@ -270,7 +307,7 @@ class Project:
         Raises
         ------
             ValueError
-                If the analysis database has already been closed, then a ValueError is raised.
+                If the project has already been closed, then a ValueError is raised.
             LookupError
                 When the analysis for the specified analysis method, category name, clustering name, and embedding name
                 could not be found, then a LookupError is raised.
@@ -281,19 +318,19 @@ class Project:
                 Returns the analysis for the specified name.
         """
 
+        if self.is_closed:
+            raise ValueError('The project has already been closed.')
+
         if analysis_method not in self.analyses:
-            raise LookupError('The specified analysis method "{0}" could not be found.'.format(analysis_method))
+            raise LookupError(f'The specified analysis method "{analysis_method}" could not be found.')
 
         for analysis_database in self.analyses[analysis_method]:
             if analysis_database.has_analysis(category_name, clustering_name, embedding_name):
                 return analysis_database.get_analysis(category_name, clustering_name, embedding_name)
 
         raise LookupError(
-            'No analysis in the category "{0}" with the clustering "{1}" and embedding "{2}" could be found.'.format(
-                category_name,
-                clustering_name,
-                embedding_name
-            )
+            f'No analysis in the category "{category_name}" with the clustering "{clustering_name}" and embedding '
+            f'"{embedding_name}" could be found.'
         )
 
     def close(self):
@@ -302,11 +339,14 @@ class Project:
         if not self.is_closed:
             if self.dataset is not None:
                 self.dataset.close()
+            self.dataset = None
             for attribution in self.attributions:
                 attribution.close()
-            for analysis_method in self.analyses:
-                for analysis in self.analyses[analysis_method]:
+            self.attributions = []
+            for _, analyses in self.analyses.items():
+                for analysis in analyses:
                     analysis.close()
+            self.analyses = {}
             self.is_closed = True
 
     def __del__(self):
@@ -332,7 +372,6 @@ class AttributionDatabase:
 
         # Initializes some class members
         self.is_closed = False
-        self.attribution_file = None
 
         # Stores the arguments for later reference
         self.attribution_path = attribution_path
@@ -345,7 +384,7 @@ class AttributionDatabase:
         # the labels are stored as a boolean NumPy array where the index is the label index and the value determines
         # whether the sample has the label, when the dataset is single-label, then the label is just a scalar value
         # containing the index of the label)
-        self.is_multi_label = self.attribution_file['label'][0].dtype == numpy.bool
+        self.is_multi_label = self.attribution_file['label'][0].dtype == bool
 
     def has_attribution(self, index):
         """
@@ -372,7 +411,7 @@ class AttributionDatabase:
 
         if 'index' in self.attribution_file.keys():
             return index in self.attribution_file['index']
-        return index <= self.attribution_file['attribution'].shape[0]
+        return index < self.attribution_file['attribution'].shape[0]
 
     def get_attribution(self, index):
         """
@@ -402,7 +441,7 @@ class AttributionDatabase:
 
         # Checks if the specified attribution exists, if not, then an LookupError is raised
         if not self.has_attribution(index):
-            raise LookupError('No attribution with the index {0} could be found.'.format(index))
+            raise LookupError(f'No attribution with the index {index} could be found.')
 
         # Check where is the searched index in the attribution file
         original_index = index
@@ -428,8 +467,8 @@ class AttributionDatabase:
         """Closes the attribution database."""
 
         if not self.is_closed:
-            if self.attribution_file is not None:
-                self.attribution_file.close()
+            self.attribution_file.close()
+            self.attribution_file = None
             self.is_closed = True
 
     def __del__(self):
@@ -451,7 +490,7 @@ class Attribution:
                 The index of the attribution, which is the index of the sample for which the attribution was created.
             data: numpy.ndarray
                 The attribution data, which is a raw heatmap.
-            labels: str or list
+            labels: Label or list
                 The ground truth label or a list of the ground truth labels of the sample for which the attribution was
                 created.
             prediction: numpy.ndarray
@@ -491,8 +530,7 @@ class Attribution:
 
         if superimpose is not None:
             return render_superimposed_heatmap(self.data, superimpose, color_map)
-        else:
-            return render_heatmap(self.data, color_map)
+        return render_heatmap(self.data, color_map)
 
 
 class AnalysisDatabase:
@@ -512,7 +550,6 @@ class AnalysisDatabase:
 
         # Initializes some class members
         self.is_closed = False
-        self.analysis_file = None
 
         # Stores the arguments for later reference
         self.analysis_path = analysis_path
@@ -690,35 +727,24 @@ class AnalysisDatabase:
         # Checks if the specified analysis exists, if not, then an LookupError is raised
         if not self.has_analysis(category_name, clustering_name, embedding_name):
             raise LookupError(
-                'No analysis for category "{0}", clustering "{1}", and embedding "{2}" could be found.'.format(
-                    category_name,
-                    clustering_name,
-                    embedding_name
-                )
+                f'No analysis for category "{category_name}", clustering "{clustering_name}", and embedding '
+                '"{embedding_name}" could be found.'
             )
 
         # Gets the analysis for the specified name, cluster, and embedding
         analysis = self.analysis_file[category_name]
-        clustering = analysis['cluster'][clustering_name]
-        embedding = analysis['embedding'][embedding_name]
-        attribution_indices = analysis['index']
-        eigen_values = None
+        clustering = analysis['cluster'][clustering_name][()]
+        embedding = analysis['embedding'][embedding_name][()]
+        attribution_indices = analysis['index'][()]
+        eigenvalues = None
         if 'eigenvalue' in self.analysis_file[category_name]['embedding'][embedding_name].attrs.keys():
-            eigen_values = self.analysis_file[category_name]['embedding'][embedding_name].attrs['eigenvalue']
-        base_embedding_name = None
-        if 'embedding' in self.analysis_file[category_name]['embedding'][embedding_name].attrs.keys():
-            base_embedding_name = self.analysis_file[category_name]['embedding'][embedding_name].attrs['embedding']
-            if 'eigenvalue' in self.analysis_file[category_name]['embedding'][base_embedding_name].attrs.keys():
-                eigen_values = self.analysis_file[category_name]['embedding'][base_embedding_name].attrs['eigenvalue']
-        base_embedding_axes_indices = None
-        if 'index' in self.analysis_file[category_name]['embedding'][embedding_name].attrs.keys():
-            base_embedding_axes_indices = self.analysis_file[category_name]['embedding'][embedding_name].attrs['index']
+            eigenvalues = self.analysis_file[category_name]['embedding'][embedding_name].attrs['eigenvalue']
 
         # Wraps the information of the analysis in an object and returns it
         try:
             human_readable_category_name = self.label_map.get_labels(category_name)
         except LookupError:
-            human_readable_category_name = category_name
+            human_readable_category_name = ''
         return Analysis(
             category_name,
             human_readable_category_name,
@@ -727,17 +753,15 @@ class AnalysisDatabase:
             embedding_name,
             embedding,
             attribution_indices,
-            eigen_values,
-            base_embedding_name,
-            base_embedding_axes_indices
+            eigenvalues
         )
 
     def close(self):
         """Closes the analysis database."""
 
         if not self.is_closed:
-            if self.analysis_file is not None:
-                self.analysis_file.close()
+            self.analysis_file.close()
+            self.analysis_file = None
             self.is_closed = True
 
     def __del__(self):
@@ -781,9 +805,7 @@ class Analysis:
             embedding_name,
             embedding,
             attribution_indices,
-            eigen_values,
-            base_embedding_name,
-            base_embedding_axes_indices
+            eigenvalues
     ):
         """
         Initializes a new Analysis instance.
@@ -796,7 +818,7 @@ class Analysis:
                 the attributions. So the category name is the umbrella term for all the attributions that comprise the
                 analysis, which will, in most cases, be the name of the label.
             human_readable_category_name: str
-                A human readable version of the category name (since category names are usually label, this can be
+                A human readable version of the category name (since category names are usually labels, this can be
                 the human-readable name of the label, if the category name is not a label, then the human-readable name
                 should be set to the category name itself).
             clustering_name: str
@@ -815,20 +837,10 @@ class Analysis:
                 analysis.
             attribution_indices: numpy.ndarray
                 Contains a list of the indices of the attributions that correspond to the embeddings and cluster points.
-            eigen_values: numpy.ndarray
+            eigenvalues: numpy.ndarray
                 The eigen values of the embedding. The eigen values must only be specified for normal embeddings that
                 are not based on another embedding (see the description for the base_embedding_name parameter for more
                 information).
-            base_embedding_name: str
-                There are two types of embeddings: normal embeddings and embeddings of embeddings. Embeddings of
-                embeddings are embeddings that are based on another embedding (e.g. the spectral embedding has a lot of
-                dimensions, so a T-SNE embedding is performed on the spectral embedding, which embeds the spectral
-                embedding in two-dimensional space). This is the name of the embedding on which the embedding is based.
-                When this is a normal embedding, then this should be None.
-            base_embedding_axes_indices: numpy.ndarray
-                The indices of the axes of the embedding on which this embedding is based. This value must only be
-                specified for embeddings that are based on other embeddings and not normal embeddings (see the
-                description for the base_embedding_name parameter for more information).
         """
 
         self.category_name = category_name
@@ -838,9 +850,7 @@ class Analysis:
         self.embedding_name = embedding_name
         self.embedding = embedding
         self.attribution_indices = attribution_indices
-        self.eigen_values = eigen_values
-        self.base_embedding_name = base_embedding_name
-        self.base_embedding_axes_indices = base_embedding_axes_indices
+        self.eigenvalues = eigenvalues
 
 
 class Hdf5Dataset:
@@ -876,7 +886,7 @@ class Hdf5Dataset:
         # the labels are stored as a boolean NumPy array where the index is the label index and the value determines
         # whether the sample has the label, when the dataset is single-label, then the label is just a scalar value
         # containing the index of the label)
-        self.is_multi_label = self.dataset_file['label'][0].dtype == numpy.bool
+        self.is_multi_label = self.dataset_file['label'][0].dtype == bool
 
     def get_sample(self, index):
         """
@@ -884,8 +894,8 @@ class Hdf5Dataset:
 
         Parameters
         ----------
-            index: int
-                The index of the sample that is to be retrieved.
+            index: int | str
+                The index or key of the sample that is to be retrieved.
 
         Raises
         ------
@@ -904,22 +914,13 @@ class Hdf5Dataset:
         if self.is_closed:
             raise ValueError('The dataset is already closed.')
 
-        # Checks if the index is out of range
-        if 'index' in self.dataset_file:
-            try:
-                data_index = self.dataset_file['index'][index].item()
-            except ValueError as error:
-                raise LookupError('No sample with the index {0} could be found.'.format(index)) from error
-        else:
-            data_index = index
-
         # Extracts the information about the sample from the dataset
         try:
-            sample_data = self.dataset_file['data'][data_index][:]
+            sample_data = self.dataset_file['data'][index][()]
             sample_label_reference = self.dataset_file['label'][index]
             sample_labels = self.label_map.get_labels(sample_label_reference)
-        except ValueError as error:
-            raise LookupError('No sample with the index {0} could be found.'.format(index)) from error
+        except IndexError as error:
+            raise LookupError(f'No sample with the index {index} could be found.') from error
 
         # Wraps the sample in an object and returns it
         return Sample(index, sample_data, sample_labels)
@@ -972,8 +973,8 @@ class Hdf5Dataset:
         """Closes the dataset."""
 
         if not self.is_closed:
-            if self.dataset_file is not None:
-                self.dataset_file.close()
+            self.dataset_file.close()
+            self.dataset_file = None
             self.is_closed = True
 
     def __del__(self):
@@ -984,7 +985,7 @@ class Hdf5Dataset:
 
 class ImageDirectoryDataset:
     """
-    Represents a dataset that is stored as image files in a directory hierarchy were the names of the directories
+    Represents an image dataset, where the image files are in a directory hierarchy were the names of the directories
     represent the labels of the images.
     """
 
@@ -1005,8 +1006,6 @@ class ImageDirectoryDataset:
 
         Parameters
         ----------
-            path: str
-                The path to the HDF5 file that contains the dataset.
             path: str
                 The path to the directory that contains the directories for the labels, which in turn contain the images
                 that belong to the respective label.
@@ -1020,10 +1019,10 @@ class ImageDirectoryDataset:
                 label_word_net_id_regex must be specified.
             input_width: int
                 The width of the input to the model. This is used to resize the dataset samples to the same size as the
-                the model expects its inputs to be.
+                model expects its inputs to be.
             input_height: int
                 The height of the input to the model. This is used to resize the dataset samples to the same size as the
-                the model expects its inputs to be.
+                model expects its inputs to be.
             down_sampling_method: str
                 The method that is to be used to down-sample images from the dataset that are larger than the input to
                 the model.
@@ -1036,6 +1035,8 @@ class ImageDirectoryDataset:
         Raises
         ------
             ValueError
+                If neither the label index RegEx nor the label WordNet ID RegEx was specified, a ValueError is raised.
+                If both the label index RegEx and the label WordNet ID RegEx were specified, a ValueError is raised.
                 If the specified up-sampling method is not supported, a ValueError is raised.
                 If the specified down-sampling method is not supported, a ValueError is raised.
         """
@@ -1044,11 +1045,15 @@ class ImageDirectoryDataset:
         self.is_closed = False
 
         # Validates the arguments
+        if label_index_regex is None and label_word_net_id_regex is None:
+            raise ValueError('Either the label index RegEx or the label WordNet ID RegEx must be specified.')
+        if label_index_regex is not None and label_word_net_id_regex is not None:
+            raise ValueError('Only the label index RegEx or the label WordNet ID RegEx must be specified, not both.')
         if down_sampling_method not in ['none', 'center_crop', 'resize']:
-            raise ValueError('The down-sampling method "{0}" is not supported.'.format(down_sampling_method))
-        if up_sampling_method not in ['none', 'fill_zeros', 'fill_ones', 'edge_repeat', 'mirror_edge', 'wrap_around',
-                                      'resize']:
-            raise ValueError('The up-sampling method "{0}" is not supported.'.format(up_sampling_method))
+            raise ValueError(f'The down-sampling method "{down_sampling_method}" is not supported.')
+        up_sampling_methods = ['none', 'fill_zeros', 'fill_ones', 'edge_repeat', 'mirror_edge', 'wrap_around', 'resize']
+        if up_sampling_method not in up_sampling_methods:
+            raise ValueError(f'The up-sampling method "{up_sampling_method}" is not supported.')
 
         # Stores the arguments for later reference
         self.name = name
@@ -1061,12 +1066,13 @@ class ImageDirectoryDataset:
         self.up_sampling_method = up_sampling_method
         self.label_map = label_map
 
-        # Loads a list of all the paths to all samples in the dataset (they are soreted, because the index of the sorted
+        # Loads a list of all the paths to all samples in the dataset (they are sorted, because the index of the sorted
         # paths corresponds to the sample index that has to be specified in the get_sample method)
         if os.path.exists(self.path + '_paths.txt'):
-            with open(self.path + '_paths.txt') as f:
-                self.sample_paths = sorted([os.path.join(os.path.dirname(self.path), path)
-                                            for path in f.read().split('\n')])
+            with open(self.path + '_paths.txt', encoding='utf-8') as f:
+                self.sample_paths = sorted(
+                    [os.path.join(os.path.dirname(self.path), path) for path in f.read().split('\n')]
+                )
         else:
             self.sample_paths = sorted(glob.glob(os.path.join(self.path, '**/*.*'), recursive=True))
         assert len(self.sample_paths) > 0, 'No Images found.'
@@ -1101,7 +1107,7 @@ class ImageDirectoryDataset:
 
         # Gets the path to the sample file
         if index >= len(self.sample_paths):
-            raise LookupError('No sample with the index {0} could be found.'.format(index))
+            raise LookupError(f'No sample with the index {index} could be found.')
         sample_path = self.sample_paths[index]
 
         # Determines the label of the sample by parsing the path
@@ -1109,7 +1115,7 @@ class ImageDirectoryDataset:
         if self.label_index_regex is not None:
             match = re.match(self.label_index_regex, sample_path)
             if match:
-                label_reference = match.groups()[0]
+                label_reference = int(match.groups()[0])
         else:
             match = re.match(self.label_word_net_id_regex, sample_path)
             if match:
@@ -1148,10 +1154,8 @@ class ImageDirectoryDataset:
         # (if for example the width is smaller than the target width but the height is larger, then the image is first
         # up-sampled so that the width matched the target width, in the next step, the image will be down-sampled, so
         # that the height also matches the target width)
-        if len(image.shape) == 2:
-            width, height = image.shape
-        else:
-            width, height, _ = image.shape
+        width = image.shape[0]
+        height = image.shape[1]
         if width < self.input_width or height < self.input_height:
             if self.up_sampling_method in ['fill_zeros', 'fill_ones', 'edge_repeat', 'mirror_edge', 'wrap_around']:
                 image = add_border(
@@ -1162,10 +1166,6 @@ class ImageDirectoryDataset:
                 )
             elif self.up_sampling_method == 'resize':
                 image = numpy.array(Image.fromarray(image).resize((self.input_width, self.input_height)))
-            elif self.up_sampling_method == 'none':
-                pass
-            else:
-                raise ValueError('The up-sampling method "{0}" is not supported.'.format(self.up_sampling_method))
 
         # If at least one of the image dimensions is greater than the target size, then the image is down-sampled
         if width > self.input_width or height > self.input_height:
@@ -1173,13 +1173,6 @@ class ImageDirectoryDataset:
                 image = center_crop(image, self.input_width, self.input_height)
             elif self.down_sampling_method == 'resize':
                 image = numpy.array(Image.fromarray(image).resize((self.input_width, self.input_height)))
-            elif self.up_sampling_method == 'none':
-                pass
-            else:
-                raise ValueError('The down-sampling method "{0}" is not supported.'.format(self.down_sampling_method))
-
-        if image.ndim == 2:  # If image black and white add the third axis
-            image = image[..., numpy.newaxis]
 
         # Returns the re-sampled image
         return image
@@ -1231,8 +1224,7 @@ class ImageDirectoryDataset:
     def close(self):
         """Closes the dataset."""
 
-        if not self.is_closed:
-            self.is_closed = True
+        self.is_closed = True
 
     def __del__(self):
         """Destructs the dataset."""
@@ -1253,7 +1245,7 @@ class Sample:
                 The index of the sample within the dataset.
             data: numpy.ndarray
                 The data that represents the sample.
-            labels: str or list
+            labels: Label or list
                 The label or a list of all the labels that the sample has.
         """
 
@@ -1285,11 +1277,7 @@ class Sample:
                 self.data *= 255.0 / 2.0
             elif detected_pixel_value_range_index == 1:
                 self.data *= 255.0
-
-            # Finally, the pixel values may be saved as floats and not as integers, so the data type is changed to unsigned
-            # 8-bit integers, which is standard for viewing images
-            if self.data.dtype != numpy.uint8:
-                self.data = self.data.astype(numpy.uint8)
+            self.data = self.data.astype(numpy.uint8)
 
 
 class LabelMap:
@@ -1310,7 +1298,7 @@ class LabelMap:
 
         # Loads the label map from the specified JSON file
         self.labels = []
-        with open(self.path, 'r') as label_map_file:
+        with open(self.path, 'r', encoding='utf-8') as label_map_file:
             for label in json.load(label_map_file):
                 self.labels.append(Label(label['index'], label['word_net_id'], label['name']))
 
@@ -1347,7 +1335,7 @@ class LabelMap:
             return self.get_label_from_index(reference.item())
         if isinstance(reference, str):
             return self.get_label_from_word_net_id(reference)
-        raise LookupError('No labels for the specified reference "{0}" could be found.'.format(reference))
+        raise LookupError(f'No labels for the specified reference "{reference}" could be found.')
 
     def get_label_from_index(self, index):
         """
@@ -1372,7 +1360,7 @@ class LabelMap:
         for label in self.labels:
             if label.index == index:
                 return label.name
-        raise LookupError('No label with the specified index {0} could be found.'.format(index))
+        raise LookupError(f'No label with the specified index {index} could be found.')
 
     def get_labels_from_n_hot_vector(self, n_hot_vector):
         """
@@ -1401,8 +1389,8 @@ class LabelMap:
             for index in numpy.argwhere(n_hot_vector):
                 labels.append(self.get_label_from_index(index[0]))
             return labels
-        except LookupError:
-            raise LookupError('One or more labels for the n-hot encoded vector do not exist.')
+        except LookupError as lookup_error:
+            raise LookupError('One or more labels for the n-hot encoded vector do not exist.') from lookup_error
 
     def get_label_from_word_net_id(self, word_net_id):
         """
@@ -1427,7 +1415,7 @@ class LabelMap:
         for label in self.labels:
             if label.word_net_id == word_net_id:
                 return label.name
-        raise LookupError('No label with the specified WordNet ID "{0}" could be found.'.format(word_net_id))
+        raise LookupError(f'No label with the specified WordNet ID "{word_net_id}" could be found.')
 
 
 class Label:
@@ -1537,7 +1525,7 @@ class Workspace:
                 return project
 
         # If no project with the specified name could not be found, then an exception is raised
-        raise LookupError('The project with the name "{0}" could not be found.'.format(name))
+        raise LookupError(f'The project with the name "{name}" could not be found.')
 
     def close(self):
         """Closes the workspace and all projects within it."""
@@ -1545,6 +1533,7 @@ class Workspace:
         if not self.is_closed:
             for project in self.projects:
                 project.close()
+            self.projects = []
             self.is_closed = True
 
     def __del__(self):
